from django.views.generic import TemplateView
from django.shortcuts import render
from django.template.loader import render_to_string
from django.http import HttpResponse

from .manager import ComponentManager


class HornetlView(TemplateView):

    def render_to_component(self, state=None):
        if not state is None:
            self.state = state
        else:
            self.state = self.component.__dict__
        html = render_to_string(f"components/{self.component_name}.html", self.state)
        return render(self.request, self.template_name, {"component_html": html})

    def update_to_component(self):
        self.manager.save_component(self.component_name, self.component)
        html = render_to_string(f"components/{self.component_name}.html", self.state)
        return HttpResponse(html)

    def dispatch(self, *args, **kwargs):
        self.manager = ComponentManager(self.request)
        self.component = self.manager.load_component(self.component_name)
        self.state = self.component.__dict__
        self.html = render_to_string(
            f"components/{self.component_name}.html", self.state
        )
        return super(HornetlView, self).dispatch(*args, **kwargs)



from django import template
from django.shortcuts import render
from django.template.loader import render_to_string
from django.http import HttpResponse

from hornet.manager import ComponentManager

register = template.Library()


@register.simple_tag(takes_context=True)
def load_component(context, app_name, component_name):
    request = context["request"]
    manager = ComponentManager(request)
    component = manager.load_component(component_name)
    state = component.__dict__
    html = render_to_string(f"components/{component_name}.html", state)
    return html



import json
from collections.abc import Awaitable
from typing import Any
from typing import Callable
from urllib.parse import unquote
from urllib.parse import urlsplit
from urllib.parse import urlunsplit

from asgiref.sync import iscoroutinefunction
from asgiref.sync import markcoroutinefunction
from django.http import HttpRequest
from django.http.response import HttpResponseBase
from django.utils.functional import cached_property


class HtmxMiddleware:
    sync_capable = True
    async_capable = True

    def __init__(
        self,
        get_response: (
            Callable[[HttpRequest], HttpResponseBase]
            | Callable[[HttpRequest], Awaitable[HttpResponseBase]]
        ),
    ) -> None:
        self.get_response = get_response
        self.async_mode = iscoroutinefunction(self.get_response)

        if self.async_mode:
            # Mark the class as async-capable, but do the actual switch
            # inside __call__ to avoid swapping out dunder methods
            markcoroutinefunction(self)

    def __call__(
        self, request: HttpRequest
    ) -> HttpResponseBase | Awaitable[HttpResponseBase]:
        if self.async_mode:
            return self.__acall__(request)
        request.htmx = HtmxDetails(request)  # type: ignore [attr-defined]
        return self.get_response(request)

    async def __acall__(self, request: HttpRequest) -> HttpResponseBase:
        request.htmx = HtmxDetails(request)  # type: ignore [attr-defined]
        return await self.get_response(request)  # type: ignore [no-any-return, misc]


class HtmxDetails:
    def __init__(self, request: HttpRequest) -> None:
        self.request = request

    def _get_header_value(self, name: str) -> str | None:
        value = self.request.headers.get(name) or None
        if value:
            if self.request.headers.get(f"{name}-URI-AutoEncoded") == "true":
                value = unquote(value)
        return value

    def __bool__(self) -> bool:
        return self._get_header_value("HX-Request") == "true"

    @cached_property
    def boosted(self) -> bool:
        return self._get_header_value("HX-Boosted") == "true"

    @cached_property
    def current_url(self) -> str | None:
        return self._get_header_value("HX-Current-URL")

    @cached_property
    def current_url_abs_path(self) -> str | None:
        url = self.current_url
        if url is not None:
            split = urlsplit(url)
            if (
                split.scheme == self.request.scheme
                and split.netloc == self.request.get_host()
            ):
                url = urlunsplit(split._replace(scheme="", netloc=""))
            else:
                url = None
        return url

    @cached_property
    def history_restore_request(self) -> bool:
        return self._get_header_value("HX-History-Restore-Request") == "true"

    @cached_property
    def prompt(self) -> str | None:
        return self._get_header_value("HX-Prompt")

    @cached_property
    def target(self) -> str | None:
        return self._get_header_value("HX-Target")

    @cached_property
    def trigger(self) -> str | None:
        return self._get_header_value("HX-Trigger")

    @cached_property
    def trigger_name(self) -> str | None:
        return self._get_header_value("HX-Trigger-Name")

    @cached_property
    def triggering_event(self) -> Any:
        value = self._get_header_value("Triggering-Event")
        if value is not None:
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                value = None
        return value



import importlib
from django.urls import resolve


class ComponentManager:
    def __init__(self, request):
        self.request = request

    def _session_key(self, name):
        return f"component_state__{name}"

    def load_component(self, name):
        try:
            app_name = str(self.request.resolver_match._func_path).split(".")[0]
            module = importlib.import_module(f"{app_name}.components.{name}")
            component_class = getattr(module, self._class_name(name))

            state = self.request.session.get(self._session_key(name), {})
            component = component_class()

            for key, value in state.items():
                setattr(component, key, value)

            return component

        except (ModuleNotFoundError, AttributeError) as e:
            raise Exception(f"Component '{name}' not found: {e}")

    def save_component(self, name, component):
        state = {
            key: getattr(component, key)
            for key in dir(component)
            if not key.startswith("__") and not callable(getattr(component, key))
        }
        if "form" in state:
            del state["form"]
        self.request.session[self._session_key(name)] = state

    def _class_name(self, name):
        return f"{name.capitalize()}Component"
